package com.ruoyi.system.service.impl;

import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.JSONWriter;
import com.ruoyi.common.core.constant.Constants;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.domain.StrFile;
import com.ruoyi.common.core.utils.DateUtils;
import com.ruoyi.common.core.utils.ServletUtils;
import com.ruoyi.common.core.utils.StringUtils;
import com.ruoyi.common.core.utils.file.FileUtils;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.common.security.service.TokenService;
import com.ruoyi.system.api.RemoteFileService;
import com.ruoyi.system.api.domain.SysFile;
import com.ruoyi.system.api.domain.SysUser;
import com.ruoyi.system.api.model.LoginUser;
import com.ruoyi.system.domain.SnServer;
import com.ruoyi.system.domain.SnServerLog;
import com.ruoyi.system.domain.SnServerTask;
import com.ruoyi.system.domain.SnTemplate;
import com.ruoyi.system.domain.dto.DeploySSLParams;
import com.ruoyi.system.domain.dto.DockerLogsParams;
import com.ruoyi.system.domain.dto.UpdateBaseParams;
import com.ruoyi.system.enums.TaskTypeEnum;
import com.ruoyi.system.kernel.CreateShellTaskKernel;
import com.ruoyi.system.kernel.DomainValidKernel;
import com.ruoyi.system.kernel.ServerLogKernel;
import com.ruoyi.system.kernel.SslCountLimit;
import com.ruoyi.system.kernel.SslNginxConfigDealKernel;
import com.ruoyi.system.mapper.SnServerLogMapper;
import com.ruoyi.system.mapper.SnServerMapper;
import com.ruoyi.system.mapper.SnServerTaskMapper;
import com.ruoyi.system.mapper.SnServerTemplateMapper;
import com.ruoyi.system.mapper.SnTemplateMapper;
import com.ruoyi.system.mq.ServerTaskSender;
import com.ruoyi.system.service.ILaFilterUaService;
import com.ruoyi.system.service.ISnServerService;
import com.ruoyi.system.service.ISysConfigService;
import com.ruoyi.system.service.ISysDictDataService;
import com.ruoyi.system.service.ISysUserService;
import com.ruoyi.system.utils.RequestUtil;
import com.ruoyi.system.utils.SSLExpirationChecker;
import com.ruoyi.system.utils.dbsync.runner.Syncer;
import com.ruoyi.system.utils.jsch.ServiceShell;
import java.util.Iterator;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;
import org.apache.commons.lang3.tuple.Pair;
import org.apache.commons.lang3.tuple.Triple;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@RefreshScope
@Service
public class SnServerServiceImpl implements ISnServerService {
   @Autowired
   private SnServerMapper snServerMapper;
   @Autowired
   private TokenService tokenService;
   @Autowired
   private SnTemplateMapper snTemplateMapper;
   @Autowired
   private SnServerTaskMapper snServerTaskMapper;
   @Autowired
   private SnServerLogMapper snServerLogMapper;
   @Autowired
   private SnServerTemplateMapper snServerTemplateMapper;
   @Autowired
   private ISysConfigService configService;
   @Autowired
   private RemoteFileService remoteFileService;
   @Autowired
   private ServerTaskSender serverTaskSender;
   @Autowired
   private DomainValidKernel domainValidKernel;
   @Autowired
   private CreateShellTaskKernel createShellTaskKernel;
   @Autowired
   private ISysDictDataService dictDataService;
   @Autowired
   private ISysUserService sysUserService;
   @Autowired
   private SslNginxConfigDealKernel sslNginxConfigDealKernel;
   @Autowired
   private SslCountLimit sslCountLimit;
   @Autowired
   private ILaFilterUaService filterUaService;
   @Value("${baseNginxUrl}")
   public String baseNginxUrl;
   @Value("${baseYamlUrl}")
   public String baseYamlUrl;
   @Value("${baseSqlUrl}")
   public String baseSqlUrl;
   @Value("${baseSshUrl}")
   public String baseSshUrl;
   @Value("${checkLicenseUrl}")
   public String checkLicenseUrl;
   @Value("${dbSync.open}")
   public Boolean dbSyncOpen;
   ExecutorService executor = Executors.newSingleThreadExecutor();

   public AjaxResult deployBasicEnv(SnServer snParam) {
      Pair<SnServerTask, String> pair = this.deployBasic(snParam);
      if (pair.getLeft() != null) {
         this.serverTaskSender.sendOrderMessage(((SnServerTask)pair.getLeft()).getTaskId());
      }

      return StringUtils.isNotEmpty((String)pair.getRight()) ? AjaxResult.error((String)pair.getRight()) : AjaxResult.success("部署中，请稍后查看服务器任务状态");
   }

   @Transactional
   public Pair<SnServerTask, String> deployBasic(SnServer snParam) {
      LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
      SnServer server = this.selectSnServerByServerId(snParam.getServerId());
      if (server == null) {
         return Pair.of((Object)null, "未查询到服务器信息");
      } else {
         String domain = snParam.getDomain();
         AjaxResult result = this.domainValidKernel.checkDomain(domain, server.getIp(), server.getServerId());
         if (!result.isSuccess()) {
            return Pair.of((Object)null, (String)result.get("msg"));
         } else {
            SnTemplate params = new SnTemplate();
            params.getParams().put("limitNum", 1);
            params.setCategory(0);
            List<SnTemplate> backs = this.snTemplateMapper.selectSnTemplateList(params);
            if (backs.isEmpty()) {
               return Pair.of((Object)null, "未获取到基础环境后台包，请联系管理员");
            } else {
               SnTemplate back = (SnTemplate)backs.get(0);
               params.setCategory(1);
               List<SnTemplate> fonts = this.snTemplateMapper.selectSnTemplateList(params);
               if (fonts.isEmpty()) {
                  return Pair.of((Object)null, "未获取到基础环境UI包，请联系管理员");
               } else {
                  SnTemplate font = (SnTemplate)fonts.get(0);
                  String dbPsd = "R8mrVY7Em85mO5O";
                  String yamlConfig = FileUtils.readFile(this.baseYamlUrl);
                  yamlConfig = yamlConfig.replace("#{sync-client-image}", back.getPackUrl());
                  StrFile strFile = new StrFile();
                  strFile.setContent(yamlConfig);
                  strFile.setFileName("docker-compose.yml");
                  strFile.setContentType("application/x-yaml");
                  R<SysFile> r = this.remoteFileService.uploadStrToMinio(strFile);
                  if (r.getCode() != Constants.SUCCESS) {
                     return Pair.of((Object)null, r.getMsg());
                  } else {
                     String yamlUrl = ((SysFile)r.getData()).getUrl();
                     String nginxConfig = FileUtils.readFile(this.baseNginxUrl);
                     nginxConfig = nginxConfig.replace("#{server_name}", domain);
                     nginxConfig = nginxConfig.replace("#{base_root}", "");
                     nginxConfig = nginxConfig.replace("#{open_80}", "");
                     strFile = new StrFile();
                     strFile.setContent(nginxConfig);
                     strFile.setFileName(domain + ".conf");
                     strFile.setContentType("text/plain");
                     R<SysFile> r1 = this.remoteFileService.uploadStrToMinio(strFile);
                     if (r1.getCode() != Constants.SUCCESS) {
                        return Pair.of((Object)null, r1.getMsg());
                     } else {
                        String nginxUrl = ((SysFile)r1.getData()).getUrl();
                        String sshConfig = FileUtils.readFile(this.baseSshUrl);
                        sshConfig = sshConfig.replace("#{docker-compose}", yamlUrl);
                        sshConfig = sshConfig.replace("#{font-back-pack}", font.getPackUrl());
                        sshConfig = sshConfig.replace("#{font-back-name}", domain);
                        sshConfig = sshConfig.replace("#{font-back-nginx}", nginxUrl);
                        strFile = new StrFile();
                        strFile.setContent(sshConfig);
                        strFile.setFileName("install.sh");
                        strFile.setContentType("text/plain");
                        R<SysFile> r2 = this.remoteFileService.uploadStrToMinio(strFile);
                        if (r2.getCode() != Constants.SUCCESS) {
                           return Pair.of((Object)null, r2.getMsg());
                        } else {
                           String sshUrl = ((SysFile)r2.getData()).getUrl();
                           SnServer upObj = new SnServer();
                           upObj.setServerId(server.getServerId());
                           upObj.setDomain(domain);
                           upObj.setVersion(this.configService.selectConfigByKey("sync-client-version"));
                           upObj.setBackVersion(back.getVersion());
                           upObj.setUiVersion(font.getVersion());
                           upObj.setUaVersion("0.0");
                           upObj.setNginxCnf(nginxUrl);
                           upObj.setStatus(1);
                           upObj.setEnDbPsd(dbPsd);
                           SnServerTask task = new SnServerTask(server, loginUser);
                           task.setTaskName("安装服务器基础环境");
                           task.setType(TaskTypeEnum.installBase.getType());
                           task.setProcess(snParam.getProcess());
                           this.snServerTaskMapper.insertSnServerTask(task);
                           if (task.getProcess() != null) {
                              upObj.setTaskId(task.getTaskId());
                           }

                           this.updateSnServer(upObj);
                           SnServerLog log = new SnServerLog(task);
                           log.setOptName("安装服务器基础环境");
                           log.setCommand("sudo curl " + sshUrl + " -o install.sh && chmod +x install.sh && sudo ./install.sh");
                           this.snServerLogMapper.insertSnServerLog(log);
                           SysUser user = this.sysUserService.selectUserById(server.getCreateId());
                           String licenseSql = "INSERT INTO la_license (code, check_url, create_time) VALUES ('" + user.getLicense() + "', '" + this.checkLicenseUrl + "', sysdate());";
                           log.setLogId((Long)null);
                           log.setOptName("插入商户许可证");
                           log.setCommand("sudo docker exec -it sync-mysql mysql -uroot -p" + dbPsd + " -D sync-client -e \"" + licenseSql + "\"");
                           this.snServerLogMapper.insertSnServerLog(log);
                           String cloakSql = "update la_user_config set cloak_key = '" + user.getCloakKey() + "',cloak_id = '" + user.getCloakId() + "' where type = 0";
                           log.setLogId((Long)null);
                           log.setOptName("更新商户斗篷配置");
                           log.setCommand("sudo docker exec -it sync-mysql mysql -uroot -p" + dbPsd + " -D sync-client -e \"" + cloakSql + "\"");
                           this.snServerLogMapper.insertSnServerLog(log);
                           if (this.dbSyncOpen) {
                              log.setLogId((Long)null);
                              log.setOptName("基础表结构同步");
                              log.setCommand("");
                              this.snServerLogMapper.insertSnServerLog(log);
                              log.setLogId((Long)null);
                              log.setOptName("更新爬虫UA库版本");
                              log.setCommand("");
                              this.snServerLogMapper.insertSnServerLog(log);
                           }

                           return Pair.of(task, (Object)null);
                        }
                     }
                  }
               }
            }
         }
      }
   }

   public AjaxResult changeDomain(SnServer snParam) {
      SnServer server = this.selectSnServerByServerId(snParam.getServerId());
      if (server == null) {
         return AjaxResult.error("未查询到服务器信息");
      } else if (!Objects.equals(server.getStatus(), 2)) {
         return AjaxResult.error("此服务器处于【" + this.dictDataService.selectDictLabel("server_status", server.getStatus().toString()) + "】状态，不能更换域名");
      } else {
         String oldDomain = server.getDomain();
         String domain = snParam.getDomain();
         if (Objects.equals(server.getDomain(), domain)) {
            return AjaxResult.warn("域名已应用到当前服务器，请勿重复操作");
         } else {
            AjaxResult result = this.domainValidKernel.checkDomain(domain, server.getIp(), (Long)null);
            if (!result.isSuccess()) {
               return result;
            } else {
               String nginxConfig = FileUtils.readFile(this.baseNginxUrl);
               nginxConfig = nginxConfig.replace("#{server_name}", domain);
               nginxConfig = nginxConfig.replace("#{base_root}", "");
               nginxConfig = nginxConfig.replace("#{open_80}", "");
               StrFile strFile = new StrFile();
               strFile.setContent(nginxConfig);
               strFile.setFileName(domain + ".conf");
               strFile.setContentType("text/plain");
               R<SysFile> r1 = this.remoteFileService.uploadStrToMinio(strFile);
               if (r1.getCode() != Constants.SUCCESS) {
                  return AjaxResult.error(r1.getMsg());
               } else {
                  String nginxUrl = ((SysFile)r1.getData()).getUrl();
                  SnServer upObj = new SnServer();
                  upObj.setServerId(server.getServerId());
                  upObj.setDomain(domain);
                  upObj.setNginxCnf(nginxUrl);
                  upObj.setSslType(0);
                  this.updateSnServer(upObj);
                  SnServerTask task = this.createShellTaskKernel.changeBaseDomain(this.snServerMapper.selectSnServerByServerId(server.getServerId()), oldDomain);
                  if (task != null) {
                     this.serverTaskSender.sendOrderMessage(task.getTaskId());
                  }

                  return AjaxResult.success("更新中，请稍后查看服务器任务状态");
               }
            }
         }
      }
   }

   public AjaxResult deployDomainSSL(DeploySSLParams params) {
      LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
      SnServer server = this.selectSnServerByServerId(params.getId());
      if (server == null) {
         return AjaxResult.error("未查询到服务器信息");
      } else if (!Objects.equals(server.getStatus(), 2)) {
         return AjaxResult.error("此服务器处于【" + this.dictDataService.selectDictLabel("server_status", server.getStatus().toString()) + "】状态，不能部署ssl证书");
      } else {
         if (params.getType() == 1) {
            AjaxResult res = this.sslCountLimit.validCount(server.getDomain(), server.getIp());
            if (!res.isSuccess()) {
               return res;
            }
         }

         if (this.snServerTaskMapper.haveDealTask(server.getServerId(), "ssl证书", "0,1") > 0) {
            return AjaxResult.error("当前服务还有未执行完的ssl证书任务，请稍后再试");
         } else {
            String domain = server.getDomain();
            SnServer upObj = new SnServer();
            upObj.setServerId(server.getServerId());
            if (params.getType() == 2) {
               upObj.setSslTime(SSLExpirationChecker.getCertificateExDate(params.getSslPem()));
            }

            String nginxConfig = FileUtils.readFile(this.baseNginxUrl);
            Triple<AjaxResult, DeploySSLParams, String> triple = this.sslNginxConfigDealKernel.deal(params, server, domain, nginxConfig);
            if (!((AjaxResult)triple.getLeft()).isSuccess()) {
               return (AjaxResult)triple.getLeft();
            } else {
               nginxConfig = (String)triple.getRight();
               params = (DeploySSLParams)triple.getMiddle();
               nginxConfig = nginxConfig.replace("#{base_root}", "");
               upObj.setSslPem(params.getSslPem());
               upObj.setSslKey(params.getSslKey());
               upObj.setSslType(params.getType());
               StrFile strFile = new StrFile();
               strFile.setContent(nginxConfig);
               strFile.setFileName(domain + ".conf");
               strFile.setContentType("text/plain");
               R<SysFile> r1 = this.remoteFileService.uploadStrToMinio(strFile);
               if (r1.getCode() != Constants.SUCCESS) {
                  return AjaxResult.error(r1.getMsg());
               } else {
                  String nginxUrl = ((SysFile)r1.getData()).getUrl();
                  upObj.setNginxCnf(nginxUrl);
                  this.updateSnServer(upObj);
                  SnServerTask task = this.createShellTaskKernel.deploySSL(this.snServerMapper.selectSnServerByServerId(server.getServerId()), params);
                  if (task != null) {
                     this.serverTaskSender.sendOrderMessage(task.getTaskId());
                     if (task.getProcess() != null) {
                        upObj = new SnServer();
                        upObj.setServerId(server.getServerId());
                        upObj.setTaskId(task.getTaskId());
                        this.updateSnServer(upObj);
                     }
                  }

                  return AjaxResult.success("部署中，请稍后查看服务器任务状态");
               }
            }
         }
      }
   }

   public AjaxResult updateUI(UpdateBaseParams params) {
      SnServer server = this.selectSnServerByServerId(params.getId());
      if (server == null) {
         return AjaxResult.error("未查询到服务器信息");
      } else if (!Objects.equals(server.getStatus(), 2)) {
         return AjaxResult.error("此服务器处于【" + this.dictDataService.selectDictLabel("server_status", server.getStatus().toString()) + "】状态，不能更新UI");
      } else {
         SnTemplate temp = new SnTemplate();
         temp.getParams().put("limitNum", 1);
         temp.setCategory(1);
         List<SnTemplate> fonts = this.snTemplateMapper.selectSnTemplateList(temp);
         if (fonts.isEmpty()) {
            return AjaxResult.error("未获取到基础环境UI包，请联系管理员");
         } else {
            SnTemplate font = (SnTemplate)fonts.get(0);
            if (Objects.equals(server.getUiVersion(), font.getVersion())) {
               return AjaxResult.error("当前已是最新版本");
            } else {
               params.setVersion(font.getVersion());
               params.setPackUrl(font.getPackUrl());
               SnServer upObj = new SnServer();
               upObj.setServerId(server.getServerId());
               upObj.setUiVersion(font.getVersion());
               this.updateSnServer(upObj);
               SnServerTask task = this.createShellTaskKernel.updateUI(this.snServerMapper.selectSnServerByServerId(server.getServerId()), params);
               if (task != null) {
                  this.serverTaskSender.sendOrderMessage(task.getTaskId());
               }

               return AjaxResult.success("部署中，请稍后查看服务器任务状态");
            }
         }
      }
   }

   public AjaxResult updateBack(UpdateBaseParams params) {
      SnServer server = this.selectSnServerByServerId(params.getId());
      if (server == null) {
         return AjaxResult.error("未查询到服务器信息");
      } else if (!Objects.equals(server.getStatus(), 2)) {
         return AjaxResult.error("此服务器处于【" + this.dictDataService.selectDictLabel("server_status", server.getStatus().toString()) + "】状态，不能更新后台版本");
      } else {
         String version = this.configService.selectConfigByKey("sync-client-version");
         if (Objects.equals(server.getVersion(), version)) {
            return AjaxResult.error("当前已是最新版本");
         } else if (!Syncer.checkConnect(server.getIp() + ":3306", server.getDeDbPsd())) {
            return AjaxResult.error("操作失败，连接客户端数据库异常，请检查是否开放3306端口");
         } else {
            SnServer upObj = new SnServer();
            upObj.setServerId(server.getServerId());
            upObj.setVersion(version);
            if (StringUtils.isEmpty(server.getUaVersion())) {
               upObj.setUaVersion("0.0");
            }

            SnTemplate tmp = new SnTemplate();
            tmp.getParams().put("limitNum", 1);
            tmp.setCategory(0);
            List<SnTemplate> backs = this.snTemplateMapper.selectSnTemplateList(tmp);
            if (backs.isEmpty()) {
               return AjaxResult.error("未获取到基础环境后台包，请联系管理员");
            } else {
               SnTemplate back = (SnTemplate)backs.get(0);
               if (!Objects.equals(server.getBackVersion(), back.getVersion())) {
                  UpdateBaseParams msParam = new UpdateBaseParams();
                  msParam.setId(server.getServerId());
                  msParam.setServeName("sync-message");
                  SnServerTask task1 = this.createShellTaskKernel.updateBack(server, msParam);
                  if (task1 != null) {
                     this.serverTaskSender.sendOrderMessage(task1.getTaskId());
                  }

                  params.setVersion(back.getVersion());
                  params.setPackUrl(back.getPackUrl());
                  params.setServeName("sync-client");
                  SnServerTask task = this.createShellTaskKernel.updateBack(this.snServerMapper.selectSnServerByServerId(server.getServerId()), params);
                  if (task != null) {
                     this.serverTaskSender.sendOrderMessage(task.getTaskId());
                     upObj.setBackVersion(back.getVersion());
                  }
               }

               SnTemplate temp = new SnTemplate();
               temp.getParams().put("limitNum", 1);
               temp.setCategory(1);
               List<SnTemplate> fonts = this.snTemplateMapper.selectSnTemplateList(temp);
               if (fonts.isEmpty()) {
                  return AjaxResult.error("未获取到基础环境UI包，请联系管理员");
               } else {
                  SnTemplate font = (SnTemplate)fonts.get(0);
                  if (!Objects.equals(server.getUiVersion(), font.getVersion())) {
                     params.setVersion(font.getVersion());
                     params.setPackUrl(font.getPackUrl());
                     SnServerTask task = this.createShellTaskKernel.updateUI(this.snServerMapper.selectSnServerByServerId(server.getServerId()), params);
                     if (task != null) {
                        this.serverTaskSender.sendOrderMessage(task.getTaskId());
                        upObj.setUiVersion(font.getVersion());
                     }
                  }

                  if (!server.getPassword().endsWith("@gn")) {
                     upObj.setEnPassword(server.getPassword());
                  }

                  if (!server.getDbPsd().endsWith("@gn")) {
                     upObj.setEnDbPsd(server.getDbPsd());
                  }

                  this.updateSnServer(upObj);
                  return AjaxResult.success("部署中，请稍后查看服务器任务状态");
               }
            }
         }
      }
   }

   public AjaxResult updateMessage(UpdateBaseParams params) {
      SnServer server = this.selectSnServerByServerId(params.getId());
      if (server == null) {
         return AjaxResult.error("未查询到服务器信息");
      } else if (!Objects.equals(server.getStatus(), 2)) {
         return AjaxResult.error("此服务器处于【" + this.dictDataService.selectDictLabel("server_status", server.getStatus().toString()) + "】状态，不能更新后台版本");
      } else {
         SnServerTask task = this.createShellTaskKernel.updateBack(server, params);
         if (task != null) {
            this.serverTaskSender.sendOrderMessage(task.getTaskId());
         }

         return AjaxResult.success("部署中，请稍后查看服务器任务状态");
      }
   }

   public AjaxResult DDLUpdate(Long serverId) {
      LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
      SnServer server = this.selectSnServerByServerId(serverId);
      if (server == null) {
         return AjaxResult.error("未查询到服务器信息");
      } else if (!Objects.equals(server.getStatus(), 2)) {
         return AjaxResult.error("此服务器处于【" + this.dictDataService.selectDictLabel("server_status", server.getStatus().toString()) + "】状态，不能更新后台版本");
      } else {
         SnServerTask task = new SnServerTask(server, loginUser);
         task.setTaskName("更新数据库表结构");
         task.setType(TaskTypeEnum.ddlUpdate.getType());
         this.snServerTaskMapper.insertSnServerTask(task);
         SnServerLog log = new SnServerLog(task);
         log.setOptName("基础表结构同步");
         log.setCommand("");
         this.snServerLogMapper.insertSnServerLog(log);
         this.serverTaskSender.sendOrderMessage(task.getTaskId());
         return AjaxResult.success();
      }
   }

   public AjaxResult syncUa(Long serverId) {
      LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
      SnServer server = this.selectSnServerByServerId(serverId);
      if (server == null) {
         return AjaxResult.error("未查询到服务器信息");
      } else if (!Objects.equals(server.getStatus(), 2)) {
         return AjaxResult.error("此服务器处于【" + this.dictDataService.selectDictLabel("server_status", server.getStatus().toString()) + "】状态，不能同步UA库");
      } else if (StringUtils.isEmpty(server.getUaVersion())) {
         return AjaxResult.error("当前版本不支持UA过滤，请先更新后台版本");
      } else {
         String version = this.configService.selectConfigByKey("client-ua-version");
         if (Objects.equals(server.getUaVersion(), version)) {
            return AjaxResult.error("当前已是最新版本");
         } else if (!Syncer.checkConnect(server.getIp() + ":3306", server.getDeDbPsd())) {
            return AjaxResult.error("操作失败，连接客户端数据库异常，请检查是否开放3306端口");
         } else {
            String[] sqlArr = this.filterUaService.generateInsertUaSql();
            if (sqlArr.length == 0) {
               return AjaxResult.error("UA库暂无数据，请联系管理员");
            } else {
               SnServerTask task = new SnServerTask(server, loginUser);
               task.setBizVersion(server.getUaVersion());
               task.setTaskName("更新爬虫UA库版本 " + version);
               task.setType(TaskTypeEnum.uaUpdate.getType());
               this.snServerTaskMapper.insertSnServerTask(task);
               SnServerLog log = new SnServerLog(task);
               log.setOptName("更新爬虫UA库版本");
               log.setCommand(server.getIp() + ":3306");
               this.snServerLogMapper.insertSnServerLog(log);
               this.serverTaskSender.sendOrderMessage(task.getTaskId());
               return AjaxResult.success("任务已提交，请稍后查看服务器任务状态");
            }
         }
      }
   }

   public String syncUa(SnServer server, Boolean delCache) {
      if (server == null) {
         return "未查询到服务器信息";
      } else {
         String version = this.configService.selectConfigByKey("client-ua-version");
         if (Objects.equals(server.getUaVersion(), version)) {
            return "当前已是最新版本";
         } else {
            String[] sqlArr = this.filterUaService.generateInsertUaSql();
            if (sqlArr.length == 0) {
               return "UA库暂无数据，请联系管理员";
            } else {
               AjaxResult res = Syncer.executeSql(server.getIp() + ":3306", server.getDeDbPsd(), sqlArr);
               if (!res.isSuccess()) {
                  return String.valueOf(res.get("msg"));
               } else {
                  SnServer upObj = new SnServer();
                  upObj.setServerId(server.getServerId());
                  upObj.setUaVersion(version);
                  this.snServerMapper.updateSnServer(upObj);
                  if (delCache) {
                     ServiceShell shell = new ServiceShell();

                     String var9;
                     try {
                        shell.connect(server.getIp(), server.getUserName(), server.getDePassword(), server.getPort());
                        shell.sendCommand("sudo docker exec -it sync-redis redis-cli DEL ua_cache_list");
                        return null;
                     } catch (Exception var13) {
                        Exception e = var13;
                        var9 = "UA库同步完成，缓存刷新失败，" + e.getMessage();
                     } finally {
                        shell.disconnect();
                     }

                     return var9;
                  } else {
                     return null;
                  }
               }
            }
         }
      }
   }

   public AjaxResult syncUaAll(List<Long> serverIds) {
      LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
      SnServer snServer = new SnServer();
      snServer.setStatus(2);
      snServer.setIsDelete(0);
      List<SnServer> servers = this.snServerMapper.selectSnServerList(snServer);
      String version = this.configService.selectConfigByKey("client-ua-version");
      servers = (List)servers.stream().filter((serverx) -> {
         if (StringUtils.isEmpty(serverx.getUaVersion())) {
            return false;
         } else {
            return !Objects.equals(serverx.getUaVersion(), version);
         }
      }).collect(Collectors.toList());
      if (servers.isEmpty()) {
         return AjaxResult.warn("没有需要更新的服务器");
      } else {
         String[] sqlArr = this.filterUaService.generateInsertUaSql();
         if (sqlArr.length == 0) {
            return AjaxResult.error("UA库暂无数据");
         } else {
            Iterator var7 = servers.iterator();

            while(var7.hasNext()) {
               SnServer server = (SnServer)var7.next();
               SnServerTask task = new SnServerTask(server, loginUser);
               task.setBizVersion(server.getUaVersion());
               task.setTaskName("更新爬虫UA库版本 " + version);
               task.setType(TaskTypeEnum.uaUpdate.getType());
               this.snServerTaskMapper.insertSnServerTask(task);
               SnServerLog log = new SnServerLog(task);
               log.setOptName("更新爬虫UA库版本");
               log.setCommand(server.getIp() + ":3306");
               this.snServerLogMapper.insertSnServerLog(log);
               this.serverTaskSender.sendOrderMessage(task.getTaskId());
            }

            return AjaxResult.success("任务已提交，请稍后查看服务器任务状态");
         }
      }
   }

   @Async
   public void updateClientCloak(Long userId) {
      try {
         SysUser common = this.sysUserService.selectUserById(userId);
         SnServer snServer = new SnServer();
         snServer.setCreateId(common.getUserId());
         snServer.setStatus(2);
         snServer.setIsDelete(0);
         List<SnServer> servers = this.snServerMapper.selectSnServerList(snServer);
         servers.forEach((server) -> {
            String url = (server.getSslType() != null && server.getSslType() != 0 ? "https://" : "http://") + server.getDomain() + "/api/user/config/updateCloak";
            JSONObject params = new JSONObject();
            params.put("license", common.getLicense());
            params.put("cloakKey", common.getCloakKey());
            params.put("cloakId", common.getCloakId());
            String res = (new RequestUtil()).doPostJson(url, params.toJSONString(new JSONWriter.Feature[0]));
            System.out.println("更新斗篷配置结果：" + res);
         });
      } catch (Exception var5) {
      }

   }

   @Async
   public void logoutMerchant(Long userId) {
      try {
         SysUser common = this.sysUserService.selectUserById(userId);
         SnServer snServer = new SnServer();
         snServer.setCreateId(common.getUserId());
         snServer.setStatus(2);
         snServer.setIsDelete(0);
         List<SnServer> servers = this.snServerMapper.selectSnServerList(snServer);
         servers.forEach((server) -> {
            String url = (server.getSslType() != null && server.getSslType() != 0 ? "https://" : "http://") + server.getDomain() + "/api/system/admin/logout";
            JSONObject params = new JSONObject();
            params.put("license", common.getLicense());
            params.put("userName", common.getUserName());
            String res = (new RequestUtil()).doPostJson(url, params.toJSONString(new JSONWriter.Feature[0]));
            System.out.println("修改密码下线客户端结果：" + res);
         });
      } catch (Exception var5) {
      }

   }

   public SnServer selectSnServerByServerId(Long serverId) {
      SnServer obj = this.snServerMapper.selectSnServerByServerId(serverId);
      if (obj != null && !obj.getIsDelete().equals(1)) {
         LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
         if (!loginUser.getRoles().contains("admin") && !loginUser.getRoles().contains("super")) {
            if (loginUser.getRoles().contains("dealer")) {
               return obj.getDealerId().equals(loginUser.getUserid()) ? obj : null;
            } else if (loginUser.getRoles().contains("agent")) {
               return obj.getAgentId().equals(loginUser.getUserid()) ? obj : null;
            } else {
               return obj.getCreateId().equals(loginUser.getUserid()) ? obj : null;
            }
         } else {
            return obj;
         }
      } else {
         return null;
      }
   }

   public List<SnServer> selectSnServerList(SnServer obj) {
      LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
      if (loginUser.getRoles().contains("dealer")) {
         obj.setDealerId(loginUser.getUserid());
      }

      if (loginUser.getRoles().contains("agent")) {
         obj.setAgentId(loginUser.getUserid());
      }

      if (loginUser.getRoles().contains("common")) {
         obj.setCreateId(loginUser.getUserid());
      }

      obj.setIsDelete(0);
      return this.snServerMapper.selectSnServerList(obj);
   }

   public AjaxResult insertSnServer(SnServer snServer) {
      LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
      if (!loginUser.getRoles().contains("admin") && !loginUser.getRoles().contains("super") && !loginUser.getRoles().contains("common")) {
         return AjaxResult.error("暂无权限添加服务器");
      } else {
         Integer num = this.snServerMapper.getServerNum(loginUser.getUserid());
         SysUser user = this.sysUserService.selectUserById(loginUser.getUserid());
         if (user.getServerLimit() != null && user.getServerLimit() <= num) {
            return AjaxResult.error("您当前最多可部署" + user.getServerLimit() + "个服务器，如需增加请联系管理员");
         } else {
            SnServer query = new SnServer();
            query.setIp(snServer.getIp());
            query.setIsDelete(0);
            if (!this.snServerMapper.selectSnServerList(query).isEmpty()) {
               return AjaxResult.error("已有相同IP的服务器，请勿重复添加");
            } else {
               snServer.setCreateId(loginUser.getUserid());
               snServer.setCreateName(loginUser.getUsername());
               if (loginUser.getRoles().contains("common")) {
                  snServer.setAgentId(user.getAgentId());
                  snServer.setAgentName(user.getAgentName());
                  snServer.setDealerId(user.getDealerId());
                  snServer.setDealerName(user.getDealerName());
               }

               snServer.setEnPassword(snServer.getPassword());
               snServer.setIsDelete(0);
               snServer.setCreateTime(DateUtils.getNowDate());
               snServer.setUpdateTime(DateUtils.getNowDate());
               snServer.setProcess(1001);
               return AjaxResult.success(this.snServerMapper.insertSnServer(snServer));
            }
         }
      }
   }

   public int updateSnServer(SnServer snServer) {
      if (this.selectSnServerByServerId(snServer.getServerId()) == null) {
         return 0;
      } else {
         try {
            LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
            snServer.setUpdateId(loginUser.getUserid());
            snServer.setUpdateName(loginUser.getUsername());
         } catch (Exception var3) {
         }

         snServer.setUpdateTime(DateUtils.getNowDate());
         snServer.setEnPassword(snServer.getPassword());
         snServer.setIp((String)null);
         snServer.setUpdateTime(DateUtils.getNowDate());
         return this.snServerMapper.updateSnServer(snServer);
      }
   }

   public int deleteSnServerByServerIds(Long[] serverIds) {
      LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
      if (!loginUser.getRoles().contains("admin") && !loginUser.getRoles().contains("super")) {
         Long createId = loginUser.getUserid();
         this.snServerTaskMapper.deleteSnServerTaskByServerIdV2(serverIds[0]);
         this.snServerTemplateMapper.deleteSnServerTemplateByServerIdsV2(serverIds);
         return this.snServerMapper.deleteSnServerByServerIdsV2(serverIds, createId);
      } else {
         this.snServerTaskMapper.deleteSnServerTaskByServerId(serverIds[0]);
         this.snServerTemplateMapper.deleteSnServerTemplateByServerIds(serverIds);
         return this.snServerMapper.deleteSnServerByServerIds(serverIds);
      }
   }

   public AjaxResult getLogs(DockerLogsParams params) {
      LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
      if (!loginUser.getRoles().contains("admin") && !loginUser.getRoles().contains("super")) {
         return AjaxResult.error("暂无权限");
      } else {
         SnServer server = this.snServerMapper.selectSnServerByServerId(params.getId());
         ServiceShell shell = new ServiceShell(server.getServerId(), (ServerLogKernel)null);
         String logs = "";

         try {
            shell.connect(server.getIp(), server.getUserName(), server.getDePassword(), server.getPort());
            if (params.getLine() == null) {
               params.setLine(100);
            }

            logs = shell.executeCommand("sudo docker logs --tail " + params.getLine() + " " + params.getServeName());
         } catch (Exception var10) {
         } finally {
            shell.disconnect();
         }

         return AjaxResult.success("获取成功", logs);
      }
   }
}
